@@ -54,7 +54,7 @@ class ConfigManager(GenericDataManager):
             "hwi_bridge_url": "/hwi/api/",
             # unique id that will be used in wallets path in Bitcoin Core
             # empty by default for backward-compatibility
-            "uid": "",
+            "uid": random.randint(0, 256**8).to_bytes(8, "big").hex(),
             "unit": "btc",
             "price_check": False,
             "alt_rate": 1,
@@ -65,7 +65,7 @@ class ConfigManager(GenericDataManager):
             "fee_estimator": "bitcoin_core",
             "fee_estimator_custom_url": "",
             "hide_sensitive_info": False,
-            "autohide_sensitive_info_timeout_minutes": 20,
+            "autohide_sensitive_info_timeout_minutes": None,
             "autologout_timeout_hours": 4,
             # TODO: remove
             "bitcoind": False,
@@ -146,7 +146,7 @@ class ConfigManager(GenericDataManager):
             chain_name = (
                 ""
                 if (chain == "main" or chain == "regtest")
-                else ("signet/" if self.chain == "signet" else "testnet/")
+                else ("signet/" if chain == "signet" else "testnet/")
             )
             explorer_data["url"] += chain_name
         # update the urls in the app config
