@@ -1,6 +1,11 @@
 import os
 import json
 import logging
+import pathlib
+from flask_babel import lazy_gettext as _
+
+from cryptoadvance.specter.devices.bitcoin_core import BitcoinCoreWatchOnly
+
 from ..helpers import alias, load_jsons
 from ..rpc import get_default_datadir
 
@@ -34,8 +39,7 @@ class DeviceManager:
             if data_folder.startswith("~"):
                 data_folder = os.path.expanduser(data_folder)
             # creating folders if they don't exist
-            if not os.path.isdir(data_folder):
-                os.mkdir(data_folder)
+            pathlib.Path(data_folder).mkdir(parents=True, exist_ok=True)
         devices = {}
         devices_files = load_jsons(self.data_folder, key="name")
         for device_alias in devices_files:
@@ -79,7 +83,7 @@ class DeviceManager:
         for device_name in self.devices:
             if self.devices[device_name].alias == device_alias:
                 return self.devices[device_name]
-        logger.error("Could not find Device %s" % device_alias)
+        logger.error(_("Could not find Device {}").format(device_alias))
 
     def remove_device(
         self,
@@ -99,25 +103,30 @@ class DeviceManager:
         return device_classes
 
     def supported_devices_for_chain(self, specter):
+        """Devices which you can create via the UI. BitcoinCoreWatchonly is not among them
+        and if we have no node, hot-wallets neither. Liquid support is limited as well.
+        """
         if not specter.chain:
-            return [
+            devices = [
                 device_class
                 for device_class in device_classes
-                if device_class.device_type != "bitcoincore"
-                and device_class.device_type != "elementscore"
+                if device_class.device_type not in ["bitcoincore", "elementscore"]
             ]
         elif specter.is_liquid:
-            return [
+            devices = [
                 device_class
                 for device_class in device_classes
                 if device_class.liquid_support
             ]
         else:
-            return [
+            devices = [
                 device_class
                 for device_class in device_classes
                 if device_class.bitcoin_core_support
             ]
+        if BitcoinCoreWatchOnly in devices:
+            devices.remove(BitcoinCoreWatchOnly)
+        return devices
 
     def delete(self, specter):
         """Deletes all the devices"""
